/**
 * This file is part of choco-solver, http://choco-solver.org/
 *
 * Copyright (c) 2023, IMT Atlantique. All rights reserved.
 *
 * Licensed under the BSD 4-clause license.
 * See LICENSE file in the project root for full license information.
 */
package org.chocosolver.solver.constraints.binary;

import org.chocosolver.solver.Model;
import org.chocosolver.solver.Solver;
import org.chocosolver.solver.constraints.Constraint;
import org.chocosolver.solver.exception.ContradictionException;
import org.chocosolver.solver.variables.BoolVar;
import org.chocosolver.solver.variables.IntVar;
import org.chocosolver.util.ESat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;
import static java.lang.System.currentTimeMillis;
import static org.chocosolver.solver.constraints.binary.element.ElementFactory.detect;
import static org.chocosolver.solver.search.strategy.Search.randomSearch;
import static org.chocosolver.util.tools.ArrayUtils.flatten;
import static org.chocosolver.util.tools.ArrayUtils.toArray;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 * Created by IntelliJ IDEA.
 * User: njussien
 * Date: 31/10/11
 * Time: 14:22
 */
public class SimilarPrefixInputTransformer43 {

    /**
     * In this case, the element factory maps the whole element constraint to
     * an arithmetic constraint (ElementFactory, line: 68) but forgets to care
     * about the value of `Index`.
     *
     * This could be fixed in constant time ensuring that
     * 0 <= index.getLB() - offset
     *
     * @throws ContradictionException never
     */
    @Test
    public void improveElement1() throws ContradictionException {
        Model choco = new Model();
        IntVar[] values = new IntVar[] { choco.intVar(new int[] { 48 }) };
        IntVar index = choco.intVar(new int[] { -4, 0 });
        IntVar value = choco.intVar(new int[] { -1, 0, 3 });
        choco.element(value, values, index, 0).post();
        choco.getSolver().propagate();
        System.out.println("values = " + Arrays.toString(values));
        System.out.println("index  = " + index);
        System.out.println("value  = " + value);
    }

    /**
     * This test is similar to `improveElement1` but shows that the upper
     * bound of the `index` variable is not taken into account either.
     *
     * This could be fixed in constant time ensuring that
     * values.length > index.getUB() - offset
     *
     * @throws ContradictionException never
     */
    @Test
    public void improveElement2() throws ContradictionException {
        Model choco = new Model();
        IntVar[] values = new IntVar[] { choco.intVar(3) };
        IntVar index = choco.intVar(new int[] { 0, 45 });
        IntVar value = choco.intVar(new int[] { -1, 0, 3 });
        choco.element(value, values, index, 0).post();
        choco.getSolver().propagate();
        System.out.println("values = " + Arrays.toString(values));
        System.out.println("index  = " + index);
        System.out.println("value  = " + value);
        // FAILS !
        assertTrue(index.isInstantiatedTo(0));
    }

    /**
     * This test is similar to the above but shows that problem also occurs
     * when the `values` array consists of one single primitive int.
     *
     * This could be fixed in constant time ensuring that
     * 0 <= index.getLB() - offset
     * and
     * values.length > index.getUB() - offset
     *
     * @throws ContradictionException never
     */
    @Test
    public void improveElement3() throws ContradictionException {
        Model choco = new Model();
        int[] values = new int[] { 3 };
        IntVar index = choco.intVar(new int[] { -7, 0, 45 });
        IntVar value = choco.intVar(new int[] { -1, 0, 3 });
        choco.element(value, values, index, 0).post();
        choco.getSolver().propagate();
        System.out.println("values = " + Arrays.toString(values));
        System.out.println("index  = " + index);
        System.out.println("value  = " + value);
        // FAILS !
        assertTrue(index.isInstantiatedTo(0));
    }

    /**
     * This test is similar to the above but shows that problem also occurs
     * when the `values` array consists of one single primitive int.
     *
     * This could be fixed in constant time ensuring that
     * 0 <= index.getLB() - offset
     * and
     * values.length > index.getUB() - offset
     *
     * @throws ContradictionException never
     */
    @Test
    public void itIsAlreadyDoneWhenThereIsMoreThanOnePrimitiveInt() throws ContradictionException {
        Model choco = new Model();
        int[] values = new int[] { 3, 56 };
        IntVar index = choco.intVar(new int[] { -7, 0, 45 });
        IntVar value = choco.intVar(new int[] { -1, 0, 3 });
        choco.element(value, values, index, 0).post();
        choco.getSolver().propagate();
        System.out.println("values = " + Arrays.toString(values));
        System.out.println("index  = " + index);
        System.out.println("value  = " + value);
        assertTrue(index.isInstantiatedTo(0));
    }
}
